/* HTTerm.c - libwww stuff for term clients */

/* Jim McBeath (jimmc@globes.com, or jimmc@alumni.caltech.edu) */

#include "HTUtils.h"
#include "HTParse.h"
#include "HTAlert.h"
#include "HTAccess.h"
#include "tcp.h"		/* Defines SHORT_NAMES if necessary */

#ifdef TERM
#include "client.h"
int HTUseTerm;

#ifdef C_BINDN
int HTCanFtpActive=1;
#else
int HTCanFtpActive=0;
#endif /* C_BINDN */
#endif

#ifdef TERM
int
HTDoConnectTerm (char *url, char *protocol, int default_port, int *s)
{
    int portnum;
    char *portstr;
    int sock;
    char line[256];
    char *p1 = HTParse(url, "", PARSE_HOST);
    int status;
    extern char *term_server;

    sprintf (line, "Making TERM/%s connection to %s.", protocol, p1);
    HTProgress (line);

    if (portstr=strchr(p1,':'))
	portnum = atol(portstr+1);
    else
	portnum = default_port;

    HTClearActiveIcon();
    sock = connect_server(term_server);
    if (sock<0) {
	free(p1);
	return -1;
    }
    if (send_command(sock, C_PORT, 0, "%s:%d", p1, portnum) < 0) {
	free(p1);
	close(sock);
	return -1;
    }

    send_command(sock, C_DUMB, 1, 0);
    *s = sock;
    free(p1);
    return 0;
}

PRIVATE int
HTConnect4pTerm(sockp,h0,h1,h2,h3,portnum)
int *sockp;
int h0,h1,h2,h3;
int portnum;
{
	int s;

	s = connect_server(0);
	if (s<0) { return -1; }
	if (send_command(s, C_PORT, 0, "%d.%d.%d.%d:%d", h0, h1, h2, h3, portnum) < 0) {
		close(s);
		return -1;
	}
	send_command(s, C_COMPRESS, 1, "n");
	send_command(s, C_DUMB, 1, 0);
	*sockp = s;
	return 0;
}

#ifdef C_BINDN	/* defined in term 1.0.8 */
int
HTDoFtpListenTerm(msp,port_command)
int *msp;	/* pointer to master_socket */
char *port_command;	/* fills it in with PORT command string */
{
	int s;
	int t;
	char port[100];	/* receive port number assigned by C_BINDN */

	s = connect_server(0);
	if (s<0) { return -1; }
	t = send_command(s, C_BINDN, 0, 0, 0);
	if (t<0) { close(s); return -1; }

	if (*msp >= 0)
		NETCLOSE(*msp);
	*msp = s;

	t = read(s,port,sizeof(port));
	if (t<0) { close(s); return -1; }
	sprintf(port_command,"PORT %s\r\n",port);

	return s;
}

int
HTDoFtpAccept(msp)
int *msp;	/* pointer to master_socket */
{
	char port[20];
	int ms = *msp;
	int s;
	int t;
	int pnum;

	t = read(ms,port,sizeof(port));
	if (t<0) {
		NETCLOSE(ms);
		*msp = -1;
		return -1;
	}
	pnum = atoi(port);
	s = connect_server(0);
	if (s<0) {return -1; }
	t = send_command(s, C_ACCEPT, 0, "%d", pnum);
	if (t<0) {close(s); return -1; }
	NETCLOSE(ms);
	*msp = s;
	return s;
}
#endif /* C_BINDN */

#endif /* TERM */

/* Open a socket and make a connection to a machine, given
 * it's 4 byte internet address and port number. */
int /* status */
HTConnect4p(sockp,h0,h1,h2,h3,portnum)
int *sockp;
int h0,h1,h2,h3;
int portnum;
{
	struct sockaddr_in soc_address;
	char *pa;
	int status;
	int s;

#ifdef TERM
	if (HTUseTerm) {
		return HTConnect4pTerm(sockp,h0,h1,h2,h3,portnum);
	}
#endif /* TERM */
	status = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (status<0) { return status; }
	s = status;

	pa = (char *)(&(soc_address.sin_addr));
	pa[0] = h0;	/* plug in the address we got back in from PASV */
	pa[1] = h1;
	pa[2] = h2;
	pa[3] = h3;
	soc_address.sin_port = htons(portnum);
	soc_address.sin_family = AF_INET;	    /* Family, host order  */

	status = connect(s, (struct sockaddr*)&soc_address,sizeof(soc_address));
	if (status<0){
		NETCLOSE(s);
		return status;			/* Bad return */
	}
	*sockp = s;
	return 0;
}

/* end */
